"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getEntityMessages = void 0;
const pluralize_1 = __importDefault(require("pluralize"));
const utils_1 = require("../../../../common/utils");
/**
 * Generate messages that should be added to app message pack (en.ts)
 * for translate entity resource name and fields.
 * Generated messages based on information from graphQL query - entity name and attributes.
 *
 * @param entityName - entity name string.
 * If entity name ends with 'Dto' or 'DTO' - this suffix will be removed in 'name' message.
 * @param attributes - graphQL attributes from graphQL query as GqlAttributeModel[], example:
 * [{
 *    name: "key",
 *    displayName: "value",
 * }]
 *
 * @returns messages as typescript object (react-admin i18n format),
 * which contains resource name and fields:
 * {
 *   Entity: {
 *     name: "Entity |||| Entities",
 *     fields: {
 *       key: "value",
 *     },
 * }
 *
 * Note: react-admin used `||||` as separator between singular and plural name forms.
 *
 */
function getEntityMessages(entityName, attributes) {
    const entityNameWithoutDTO = utils_1.removeDtoSuffixFromEntityName(entityName);
    return {
        [entityName]: {
            name: `${entityNameWithoutDTO} |||| ${pluralize_1.default(entityNameWithoutDTO)}`,
            fields: attributes.reduce((acc, attr) => {
                // TODO which format if typeof attr.name !== "string"
                // test needed for this case
                const key = typeof attr.name === "string" ? attr.name : attr.name.join(".");
                return { ...acc, [key]: attr.displayName };
            }, {}),
        },
    };
}
exports.getEntityMessages = getEntityMessages;
//# sourceMappingURL=getEntityMessages.js.map